! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! File:      omsourceparameter_mod
! Author:    Jorgo Bakker (jbakker@rssd.esa.int)
! Generated: Fri Aug  9 16:47:44 MEST 2002
! Usage:     C++/F95
! Example:
!   type(OmSourceParameter)          :: mySource
!   type(OmSourceParameter), pointer :: mySources(:)
!
!   call construct(mySource)         ! init contents
!   call new(mySources,low=0,high=5) ! allocate(s(0:5) and init...
!   :
!   call delete(mySources)
!   call destruct(mySource)
!use omdetectdetection_mod
module omgsourceparameter_mod
   use types
   use cal
   use dal
   use errorHandling

   use omsourceutils_mod
   use omsourceparam_mod
   implicit none

public

 character(*), parameter :: myModule="omsourceparameter_mod"






    !**********************************************************************
    ! The following structure stores the parameters for a detected source
    !**********************************************************************

    type GSourceParameters
        real(double)   :: rightAscension = -1.0     ! Right Ascension
        real(double)   :: declination    = -1.0     ! declination     
        real(single)   :: xPos           = -1.0     ! x source pos (pixels)
        real(single)   :: yPos           = -1.0     ! y source pos (pixels)
        real(single)   :: poserr = -1.0 
        real(single)   :: semiA = nullValue         ! width of psf
        real(single)   :: semiB = nullValue         ! width of psf
        real(single)   :: theta  = 0.0              ! The position-angle for each source     
        integer(int32) :: noSourcePixels = 0     ! The number of pixels included by a source
        integer(int32) :: width = 0
        integer(int32) :: height = 0
        integer(int32) :: leftOffset = 0
        integer(int32) :: leftWidth = 0
        integer(int32) :: rightOffset = 0
        integer(int32) :: rightWidth = 0
        integer(int32) :: spectrumNo = 0
        integer(int32) :: origin = 0
        integer(int16)  :: qualityFlag  = 0       ! Set to 1 if 1 or more source pixels is bad
        integer(int8)  :: extensionFlag   = 0    ! Point-source (0), otherwise extended (1)
        integer(int8)  :: confusionFlag  = 0     !
      
    end type GSourceParameters
    
  





    type PhotometryParameters
        real(single)    :: apertureRadius              ! cal aperture radius (binned pixels)
  
        real(single)    :: calApertureRadius              ! cal aperture radius (binned pixels)
    end type PhotometryParameters 
  

    !*****************************************************
    ! The observation details
    !*****************************************************
    type ObsDetails
        character(40) :: telescope
        character(40) :: observer
        character(40) :: object
        character(40) :: instrument
        character(40) :: obsID
        character(40) :: expID
        character(40) :: startObs
        character(40) :: endObs
        character(40) :: filter
        real(single)  :: exposure
    end type ObsDetails


    !*****************************************************
    ! The observation details
    !*****************************************************
    type ImageWindow
        integer(int32) :: x0
        integer(int32) :: y0
        real(double)   :: RA
        real(double)   :: Dec  
        real(double)   :: frameTime
        real(double)   :: deadFraction
        integer(int32) :: width
        integer(int32) :: height
        integer(int32) :: bx
        integer(int32) :: by
        logical        :: binBpe
        character(40)  :: filter
        character(40)  :: datamode
        real(double)   :: plateScale
    end type ImageWindow




  type OmCelestial
!      private
      integer(int32) :: x0,y0   ! offset in image pixels
      integer(int32) :: bx,by   ! binning in image pixels
      real(double)   :: plateScale       
      real(double)   :: rollAngle       
      real(double)   :: ra,dec  ! of centre point

      ! info as above, but shortcuts, that are heavily used
      real(double)   :: d_x0,d_y0
      real(double)   :: d_bx,d_by
      real(single)   :: sinRoll,cosRoll ! of rollangle
   end type OmCelestial


   ! local storage types:
   type(ImageWindow), public      :: myImageWindow
   type(ObsDetails), public       :: myObsDetails
   type(OmCelestial), public      :: myCelestial

   real(single),parameter, public :: myNullValue=99999.

   ! image related stuff:
   real(single), pointer, public  :: myImage(:, :)   ! spatial
   logical(bool),pointer, public  :: myQuality(:, :) ! quality array
  


  
   real(single), public           :: myBgrMode
   real(single), public           :: myBgrModeSigma

   real(single), public           :: myExposure
  
 

   ! Some CCD parameters for computing magnitudes:
   real(double), public           :: myDeadFraction
   real(double), public           :: myFrameTime
   real(single), public           :: myDetectionRate
   real(single), public           :: myDetectionMagnitude


   ! The number of current spectra
   integer(int32), public         :: mySpectrumNo = 0  

   ! datasets in memory:
   type(DataSetT), public         :: myImageSet

   ! determine whether we loaded sources or not?
   ! TODO: this is used for myDetectionRate/Magnitude, but
   ! if the user starts adding sources, is this still valid?
   logical, public                :: mySourcesAreLoaded
   





!   private  ! all, but (see for more information below)

   public  :: SourceParameters
   public  :: PhotometryParameters
   public :: ObsDetails   ! NB
   public :: ImageWindow
   public  :: construct
   public  :: destruct
   public  :: new
   public  :: delete

  




!   interface construct
!      module procedure construct_od
!   end interface

!   interface destruct
!      module procedure destruct_od
!   end interface
  

   interface construct
      module procedure construct_
   end interface

   interface destruct
      module procedure destruct_
   end interface

   interface new
      module procedure new_
   end interface

   interface delete
      module procedure delete_
   end interface






 
contains

   subroutine construct_(s)
      type(GSourceParameters), intent(out) :: s
      s%xpos=0

   end subroutine construct_



   subroutine destruct_(s)
      type(GSourceParameters),intent(inout) :: s
      ! we don't have to do something in here 
      ! but keep the compiler happy
      s%xPos=0
   end subroutine destruct_



   subroutine new_(sources, low, high)
      type(GSourceParameters),pointer    :: sources(:)
      integer,                intent(in) :: low,high

      integer                         :: i, status
      allocate(sources(low:high),stat=status)
!      call assertOn(status/=0,myModule//".new.sources")
      do i = low, high
         call construct(sources(i))
      end do
   end subroutine new_







   subroutine delete_(sources)
      type(GSourceParameters), pointer :: sources(:)

      if (associated(sources)) then


         deallocate(sources)
         nullify(sources)
      end if
   end subroutine delete_



end module omgsourceparameter_mod
